/* wb_afficheur.c
 * Projet WhiteBoard: afficheur en mode texte
 * Auteur : VOURIOT Thierry
 * Date : 27 decembre 2001
 */

#include "wb.h"

/* Memoire partagee (mur + pids) */
static MurAffs *smur;

/* Fonction qui affiche un message d'erreur et quitte le programme */
void erreur(char *s)
{
	perror(s);
	exit(-1);
}

/* Handler affichant le mur de caracteres */
void aff(int s)
{
	int i;
	if (signal(SIGALRM,aff) < 0)
		erreur("Erreur primitive signal");

	if (signal(SIGTERM,aff) < 0)
		erreur("Erreur primitive signal");
	
	alarm(AFFICHEUR_TEMPO);
	
	printf("\033[H");
	for (i=1;i<=MUR_TAILLE;i++)
	{
		printf("%c",smur->mur[i-1]);
		if (i % MUR_LONGUEUR == 0)
			printf("\n");
	}
}

/* Handler quittant le programme (signal SIGINT) */
void quit(int n)
{
	int i;
	pid_t mypid = getpid();
	/* On enleve l'afficheur de la liste des pid des afficheurs */
	for (i=0;i<AFFICHEUR_NBMAX;i++)
		if (smur->afficheurs[i] == mypid)
			{smur->afficheurs[i] = -1;break;}
	exit(0);
}

/* Main:  affiche le mur de carateres de la memoire partagee
 * a intervalles reguliers et a reception d'un signal SIGTERM
 */
int main(int argc, char **argv)
{
	int id;
	int i,pid;
	key_t cle;

	if (argc != 2) {
		printf("Usage: wb_afficheur shmcle\n");
		return -1;
	}	
	if (sscanf(argv[1],"%d",&cle) == 0) {
		printf("Cle invalide: echec\n");
		return -1;
	}	
	if ((id = shmget(cle,sizeof(int),0)) == -1)
		erreur("Le segment de memoire specifie n'existe pas\n");

	if ((smur = (MurAffs *)shmat(id,0,0)) < 0)
		erreur("Erreur primitive shmat");

	if (signal(SIGALRM,aff) < 0)
		erreur("Erreur primitive signal");

	if (signal(SIGTERM,aff) < 0)
		erreur("Erreur primitive signal");

	if (signal(SIGINT,quit) < 0)
		erreur("Erreur primitive signal");

	/* On effectue un clear */
	if ((pid = fork()) < 0)
		erreur("Erreur lors de la creation du processus fils");

	if (pid == 0) {		
		execlp("clear","clear",NULL);
	}
	wait(NULL);

	/* Ajout du pid dans la liste des pid des afficheurs */
	for (i=0;i<AFFICHEUR_NBMAX;i++)
	{
		if (smur->afficheurs[i] == -1)
		{
			smur->afficheurs[i] = getpid();
			break;
		}
	}
	
	alarm(1);
	while (1)
		pause();

	return 0;
}
