/* wb_gafficheur.c
 * Projet WhiteBoard: afficheur en mode graphique
 * Auteur : VOURIOT Thierry
 * Date : 28 decembre 2001
 */

#include "wb.h"
#include <gtk/gtk.h>

/* Memoire partagee (mur + pids) */
static MurAffs *smur;
/* Label ou est affiche le mur */
static GtkWidget *label;

/* Fonction qui affiche un message d'erreur et quitte le programme */
void erreur(char *s)
{
	perror(s);
	exit(-1);
}

/* Handler quittant le programme (signal SIGINT) */
void quit(int n)
{
	int i;
	pid_t mypid = getpid();
	/* On enleve l'afficheur de la liste des pid des afficheurs */
	for (i=0;i<AFFICHEUR_NBMAX;i++)
		if (smur->afficheurs[i] == mypid)
			{smur->afficheurs[i] = -1;break;}
	exit(0);
}

/* Handler permettant la fermeture de la fenetre */
void fenetreDestroy(GtkWidget *fenetre)
{
	g_print("A bientot !!\n");
	gtk_main_quit();
	quit(0);
}

/* Handler affichant le mur de caracteres */
void aff(int s)
{
	int i,k;
	char murchaine[MUR_TAILLE+MUR_HAUTEUR+1];
	if (signal(SIGALRM,aff) < 0)
		erreur("Erreur primitive signal");

	if (signal(SIGTERM,aff) < 0)
		erreur("Erreur primitive signal");
	
	alarm(AFFICHEUR_TEMPO);
	
	k=0;
	for (i=1;i<=MUR_TAILLE;i++)
	{
		murchaine[k] = smur->mur[i-1];
		k++;
		if (i % MUR_LONGUEUR == 0)
		{
			murchaine[k] = '\n';
			k++;
		}
	}
	murchaine[k] = '\0';
	gtk_label_set((GtkLabel *)label,murchaine);
}

/* Main:  affiche le mur de carateres de la memoire partagee
 * a intervalles reguliers et a reception d'un signal SIGTERM
 */
int main(int argc, char **argv)
{
	GtkWidget *fenetre;
	
	int id;
	int i;
	key_t cle;

	if (argc != 2) {
		printf("Usage: wb_gafficheur shmcle\n");
		return -1;
	}	
	if (sscanf(argv[1],"%d",&cle) == 0) {
		printf("Cle invalide: echec\n");
		return -1;
	}	
	if ((id = shmget(cle,sizeof(int),0)) == -1)
		erreur("Le segment de memoire specifie n'existe pas\n");

	if ((smur = (MurAffs *)shmat(id,0,0)) < 0)
		erreur("Erreur primitive shmat");

	if (signal(SIGALRM,aff) < 0)
		erreur("Erreur primitive signal");

	if (signal(SIGTERM,aff) < 0)
		erreur("Erreur primitive signal");

	if (signal(SIGINT,quit) < 0)
		erreur("Erreur primitive signal");

	/* Ajout du pid dans la liste des pid des afficheurs */
	for (i=0;i<AFFICHEUR_NBMAX;i++)
	{
		if (smur->afficheurs[i] == -1)
		{
			smur->afficheurs[i] = getpid();
			break;
		}
	}
	
	/* Initialisation de la boite a outils Gtk+ */
	gtk_init(&argc,&argv);

	/* Creation de la fenetre principale */
	fenetre = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	/* Taille de la fenetre */
	gtk_widget_set_usize(fenetre,600,415);

	/* Connection du signal emis par le serveur X
	avec la fonction de rappel */
	gtk_signal_connect(GTK_OBJECT(fenetre),"destroy",
	GTK_SIGNAL_FUNC(fenetreDestroy),NULL);

	/* Creation du label */
	label = gtk_label_new("");
	
	/* Ajouter le label dans la fenetre */
	gtk_container_add(GTK_CONTAINER(fenetre),label);

	/* Rendre la fenetre principale visible */
	gtk_widget_show_all(fenetre);

	alarm(1);	

	/* Entrer dans la boucle des evenements */
	gtk_main();

	return 0;
}

