/* wb_serveur.c
 * Projet WhiteBoard: serveur
 * Auteur : VOURIOT Thierry
 * Date : 27 decembre 2001
 */

#include "wb.h"

/* Memoire partagee (mur + pids) */
static MurAffs *smur;
/* Identifiant de la memoire partagee */
static int id;

/* Fonction qui affiche un message d'erreur et quitte le programme */
void erreur(char *s)
{
	perror(s);
	exit(-1);
}

/* Handler affichant la liste des afficheurs toutes les 5 secondes*/
void aff(int n)
{
	int i;
	signal(SIGALRM,aff);
	alarm(5);	
	printf("\nListe des afficheurs\n");
	for (i=0;i<10;i++)
		printf("Afficheur %d : %d\n",i+1,smur->afficheurs[i]);
}

/* Handler quittant le programme (signal SIGINT) 
 * et detruisant la memoire partagee 
 */
void quit(int n)
{
	if (shmctl(id,IPC_RMID,NULL) == -1)
		erreur("Erreur lors de la destruction de la memoire");
	exit(0);
}

/* Handler notifiant a tout les afficheurs que le mur a change */
void maj(int n)
{
	int i;
	if (signal(SIGTERM,maj) < 0)
		erreur("Erreur primitive signal");
	for (i=0;i<AFFICHEUR_NBMAX;i++)
		if (smur->afficheurs[i] != -1)
			kill(smur->afficheurs[i],SIGTERM);
				
	printf("maj: signal SIGTERM envoye aux afficheurs\n");
}

/* Main: tient a jour une liste d'afficheurs et les previent des maj */
int main(int argc, char** argv)
{
	key_t cle;
	int i;
	
	if (argc != 2) {
		printf("Usage: wb_serveur shmcle\n");
		return -1;
	}	
	if (sscanf(argv[1],"%d",&cle) == 0) {
		printf("Cle invalide: echec\n");
		return -1;
	}	
	if ((id = shmget(cle,sizeof(MurAffs),IPC_CREAT | 0666)) == -1)
		erreur("Echec lors creation de la memoire");

	if ((smur = (MurAffs *)shmat(id,0,0)) < 0)
		erreur("Erreur primitive shmat");

	/* initialisation du mur de caracteres */
	for (i=0;i<2000;i++)
		smur->mur[i] = '_';

	smur->serveur = getpid();

	/* Initialisation de la liste des afficheurs */
	for (i=0;i<AFFICHEUR_NBMAX;i++)
		smur->afficheurs[i] = -1;
	
	if (signal(SIGINT,quit) < 0)
		erreur("Erreur primitive signal");

	if (signal(SIGALRM,aff) < 0)
		erreur("Erreur primitive signal");

	if (signal(SIGTERM,maj) < 0)
		erreur("Erreur primitive signal");

	alarm(1);
	while(1)
		pause();

	return 0;
}
